#include "msimpledrawer.h"
#include <QMouseEvent>
#include <QDebug>
#include <QToolTip>
mSimpleDrawer::mSimpleDrawer(double xmin, double xmax,
                               double ymin, double ymax,
                               QWidget *parent) :
    QWidget(parent),
    xMax(xmax), xMin(xmin),
    yMax(ymax), yMin(ymin)
{
    label = new QLabel(this);

    label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    label->resize(size());

    pixMap = new QPixmap(this->size());
    Clean();
}

mSimpleDrawer::mSimpleDrawer(QWidget *parent):
    QWidget(parent),
    xMax(10), xMin(-10),
    yMax(10), yMin(-10)
{
    label = new QLabel(this);
    label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    label->resize(size());

    pixMap = new QPixmap(this->size());
    label->setMouseTracking(true);
    Clean();
}

mSimpleDrawer::~mSimpleDrawer()
{
    delete pixMap;
    delete label;
}

void mSimpleDrawer::paintEvent(QPaintEvent * e)
{
    label->resize(size());
    QSize s = label->size();

#if QT_VERSION < QT_VERSION_CHECK(4,8,0)
    QPixmap *tmp = new QPixmap((pixMap->scaled(s,Qt::IgnoreAspectRatio,Qt::SmoothTransformation)));
    delete pixMap;
    pixMap = tmp;
#else
    QPixmap tmp((pixMap->scaled(s,Qt::IgnoreAspectRatio,Qt::SmoothTransformation)));
    pixMap->swap(tmp);
#endif
    label->setPixmap(*pixMap);

}

void mSimpleDrawer::mouseMoveEvent(QMouseEvent *event)
{
   double x = event->x();//Recuperer les coordonnees de la souris dans widgetCameleon
   double y = event->y();
   emit drawArtRob(xPixToX(x),yPixToY(y));
   QToolTip::showText(event->globalPos(),
                          //  In most scenarios you will have to change these for
                          //  the coordinate system you are working in.
                          QString::number( xPixToX(event->pos().x()) ) + ", " +
                          QString::number( yPixToY(event->pos().y()) ),
                          this, rect() );
       QWidget::mouseMoveEvent(event);  // Or whatever the base class is.
}

void mSimpleDrawer::mousePressEvent(QMouseEvent *event)
{
    double x = event->x();//Recuperer les coordonnees de la souris dans widgetCameleon
    double y = event->y();
    if (event->button() == Qt::LeftButton)
        emit addPoint(xPixToX(x),yPixToY(y), true);
    else if(event->button() == Qt::RightButton)
        emit addPoint(xPixToX(x),yPixToY(y), false);

}


void mSimpleDrawer::resizeEvent(QResizeEvent *)
{
    label->resize(size());
    QSize s = label->size();
#if QT_VERSION < QT_VERSION_CHECK(4,8,0)
    QPixmap *tmp = new QPixmap((pixMap->scaled(s,Qt::IgnoreAspectRatio,Qt::SmoothTransformation)));
    delete pixMap;
    pixMap = tmp;
#else
    QPixmap tmp((pixMap->scaled(s,Qt::IgnoreAspectRatio,Qt::SmoothTransformation)));
    pixMap->swap(tmp);
#endif

}


double mSimpleDrawer::xToPix(double x)
{
    double echx = pixMap->width()/(xMax-xMin);
    return (x-xMin)*echx;
}

double mSimpleDrawer::yToPix(double y)
{
    double echy = pixMap->height()/(yMax - yMin);
    return pixMap->height() - (y-yMin)*echy;
}
double mSimpleDrawer::xPixToX(double px){
    double echx = pixMap->width()/(xMax-xMin);
    return xMin + px/echx;
}
double mSimpleDrawer::yPixToY(double py){
    double echy = pixMap->height()/(yMax - yMin);
    return yMin + (pixMap->height()-py)/echy;
}

void mSimpleDrawer::Clean()
{
    QPainter p(pixMap) ;
    QRect dim = p.window() ;
    p.setPen(QPen(Qt::red, 4)) ;
    p.setBrush(QBrush(Qt::white)) ;
    p.drawRect(dim) ;
}

void mSimpleDrawer::Center(double x, double y)
{
    double Rx=(xMax-xMin)/2.0;
    double Ry=(yMax-yMin)/2.0;
    xMin=x-Rx;    xMax=x+Rx;
    yMin=y-Ry;    yMax=y+Ry;
}

void mSimpleDrawer::DrawArrow(double x, double y,
                               double dx, double dy,
                               double r, QPen pen)
{
    double x2=x+dx;
    double y2=y+dy;
    double a=3.14-1.0;
    double px=x2+r*(cos(a)*dx-sin(a)*dy);    // cot de pointe
    double py=y2+r*(sin(a)*dx+cos(a)*dy);
    double qx=x2+r*(cos(-a)*dx-sin(-a)*dy);  // autre cot de pointe
    double qy=y2+r*(sin(-a)*dx+cos(-a)*dy);

    QPainter p(pixMap);
    p.setPen(pen);
    p.drawLine(xToPix(x),yToPix(y),xToPix(x2),yToPix(y2));
    p.drawLine(xToPix(x2),yToPix(y2),xToPix(px),yToPix(py));
    p.drawLine(xToPix(x2),yToPix(y2),xToPix(qx),yToPix(qy));
}

void mSimpleDrawer::DrawBox(double xmin, double xmax,
                             double ymin, double ymax,
                             QPen pen, QBrush brush)
{
    QPolygonF box;
    box << QPointF(xToPix(xmin),yToPix(ymin))
         << QPointF(xToPix(xmax),yToPix(ymin))
         << QPointF(xToPix(xmax),yToPix(ymax))
         << QPointF(xToPix(xmin),yToPix(ymax));

    QPainter p(pixMap);
    p.setPen(pen);
    p.setBrush(brush);
    p.drawPolygon(box);
    update();
}

void mSimpleDrawer::DrawEllipse(double cx, double cy, double r,
                                 QPen pen, QBrush brush)
{
    QPainter p(pixMap);
    p.setPen(pen);
    p.setBrush(brush);
    p.drawEllipse(xToPix(cx-r),yToPix(cy+r),
                  xToPix(cx+r)-xToPix(cx-r),yToPix(cy-r)-yToPix(cy+r));
}

void mSimpleDrawer::DrawLine(double x1, double y1,
                              double x2, double y2,
                              QPen pen)
{
    QPainter p(pixMap);
    p.setPen(pen);
    p.drawLine(xToPix(x1),yToPix(y1),xToPix(x2),yToPix(y2));
}

void mSimpleDrawer::DrawPolygon(double x, double y,
                                  double theta,
                                  vector<double> X, vector<double> Y,
                                  QPen pen, QBrush brush)
{
    QPolygonF poly;
    for (int k=0;k<X.size();k++)
    {
        double x1=x+cos(theta)*X[k]-sin(theta)*Y[k];
        double y1=y+sin(theta)*X[k]+cos(theta)*Y[k];
        poly << QPointF(xToPix(x1),yToPix(y1));
    }

    QPainter p(pixMap);
    p.setPen(pen);
    p.setBrush(brush);
    p.drawPolygon(poly);
}

void mSimpleDrawer::DrawRobot(double x, double y, double theta)
{
    vector<double> X,Y;
    X.push_back(0); Y.push_back(-1);
    X.push_back(3); Y.push_back(0);
    X.push_back(0); Y.push_back(1);
    DrawPolygon(x,y,theta,X,Y, QPen(Qt::blue), QBrush(Qt::black));
}

void mSimpleDrawer::DrawSailboat(double x, double y,
                                  double theta, QColor colr,
                                  double deltag, QColor colg,
                                  double deltav, QColor colv)
{
    double a=0.8;
    vector<double> Xc,Yc;                //coque
    Xc.push_back(-a);  Yc.push_back(-a);
    Xc.push_back(4*a); Yc.push_back(0);
    Xc.push_back(-a);  Yc.push_back(a);
    Xc.push_back(-a);  Yc.push_back(-a);
    DrawPolygon(x,y,theta,Xc,Yc,colr, colr);

    vector<double> Xr,Yr;  //gouvernail
    Xr.push_back(-2*a);  Yr.push_back(0);
    Xr.push_back(0);   Yr.push_back(0);
    DrawPolygon(x-a*cos(theta),y-a*sin(theta),
                 theta+deltag,Xr,Yr,QPen(colg),QBrush(colg));

    vector<double> Xv,Yv;  //voile
    Xv.push_back(-5*a);  Yv.push_back(0);
    Xv.push_back(0);     Yv.push_back(0);
    DrawPolygon(x+3*a*cos(theta),y+3*a*sin(theta),
                 theta+deltav,Xv,Yv,colv,colv);

}

void mSimpleDrawer::DrawText(double x, double y,
                              QString s, QColor col)
{
    QPainter p(pixMap);
    p.setPen(QPen(col));
    p.setFont(QFont("Arial"));
    p.drawText((int)xToPix(x),(int)yToPix(y),s);
}

void mSimpleDrawer::Save(QString s)
{
    pixMap->save(s+".png","PNG",100);
}
void mSimpleDrawer::Load(QString s){
    if(!pixMap->load(s)){
        qDebug() << "erreur chargement de la carte";
    }
}

void mSimpleDrawer::setDrawingArea(double xmin, double xmax,
                                   double ymin, double ymax)
{
    xMax=xmax;
    xMin=xmin;
    yMax=ymax;
    yMin=ymin;
}

//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
QPolygonF mSimpleDrawer::poly2Qpoly(Polygon &po){
    QPolygonF poly;
    for(uint j = 0; j < po.size(); j++){
        poly << QPointF(xToPix(po[j].X/1000.0), yToPix(po[j].Y/1000.0));
    }
    return poly;
}
//--------------------------------------------------------------------------------------------------
Polygon mSimpleDrawer::box2Poly(box &X0){
    Polygon p;
    box X = 1000*X0;
    p.push_back(IntPoint(X[1].inf,X[2].inf));
    p.push_back(IntPoint(X[1].inf,X[2].sup));
    p.push_back(IntPoint(X[1].sup,X[2].sup));
    p.push_back(IntPoint(X[1].sup,X[2].inf));
    return p;
}
//--------------------------------------------------------------------------------------------------
int mSimpleDrawer::DrawClippedBox(box &Xold, box &X, QPen pen1, QBrush brush1){
    Polygon subj, clip;
    Polygons solution;


    subj = box2Poly(Xold);
    clip = box2Poly(X);

    //perform intersection ...
    Clipper c;
    c.AddPolygon(subj, ptSubject);
    c.AddPolygon(clip, ptClip);
    c.Execute(ctDifference, solution, pftNonZero, pftNonZero);

    for(int i = 0; i < solution.size(); i++){
        Polygon po = solution[i];
        QPolygonF poly = poly2Qpoly(po);
        QPainter p(pixMap);
        p.setPen(pen1);
        p.setBrush(brush1);
        p.drawPolygon(poly);
    }
    return solution.size();
}

//--------------------------------------------------------------------------------------------------
int mSimpleDrawer::DrawBoxDiff(box &Xold, box &X, QPen pen1, QBrush brush1){
    int nb_box_drawed = 0;
    if(SubsetStrict(X,Xold) == false){
        nb_box_drawed = DrawClippedBox(Xold,X,pen1,brush1);
    } else {
        vector<box>* dif = diff(Xold,X);
        for(int i = 0; i  < dif->size(); i++){
            box Xt = dif->at(i);
            DrawBox(Xt[1].inf,Xt[1].sup,Xt[2].inf,Xt[2].sup, pen1, brush1);
        }
        nb_box_drawed = dif->size();
        delete dif;
    }
    return nb_box_drawed;
}
