#include "mainwindow.h"
#include "ui_mainwindow.h"


//=================================================================
//========================== PARAMETERS ===========================
//=================================================================
double xmin,xmax,ymin,ymax;
double epsilon;
double xv,yv,thetav;
double d1 = 4.5; double d2 = 1.8; double delta = 1;
vector<double> murs_xa,murs_xb,murs_ya,murs_yb;

double lim = 5*M_PI_4;
double sxmin = -lim;
double sxmax = lim;
double symin = -lim;
double symax = lim;
//=================================================================
//=================================================================


MainWindow::MainWindow(QWidget *parent) :  QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->Rsivia->setMouseTracking(true);

    xmin=-10;xmax=10;    ymin=-10;ymax=10;

    Rworld=new repere(this,ui->graphicsView_World,-10,10,-10,10);

    sivia = new Sivia(ui->Rsivia, ui->EpsilonSpinBox->value());

    // Initialize RSivia window
    ui->Rsivia->setMouseTracking(true);
    ui->Rsivia->Clean();
    ui->Rsivia->setDrawingArea((sxmin-0.1),(sxmax+0.1),(symin-0.1),(symax+0.1));
    ui->Rsivia->DrawBox((sxmin-0.1),(sxmax+0.1),(symin-0.1),(symax+0.1), QPen(Qt::white), QBrush(Qt::white));
//    ========              HARD CODED Map                              ===================================
    murs_xa.resize( 13 );  murs_xb.resize( 13 );  murs_ya.resize( 13 );  murs_yb.resize( 13 );
    murs_xa[ 0 ] =  -5.5 ; murs_ya[ 0 ] =  -1.5 ;  murs_xb[ 0 ] =  -6.40625 ; murs_yb[ 0 ] =  -1.5625 ;
    murs_xa[ 1 ] =  -6.40625 ; murs_ya[ 1 ] =  -1.5625 ;  murs_xb[ 1 ] =  -5.90625 ; murs_yb[ 1 ] =  -3.5625 ;
    murs_xa[ 2 ] =  -5.90625 ; murs_ya[ 2 ] =  -3.5625 ;  murs_xb[ 2 ] =  -2.40625 ; murs_yb[ 2 ] =  -5.0625 ;
    murs_xa[ 3 ] =  -2.40625 ; murs_ya[ 3 ] =  -5.0625 ;  murs_xb[ 3 ] =  3.59375 ; murs_yb[ 3 ] =  -5.0625 ;
    murs_xa[ 4 ] =  3.59375 ; murs_ya[ 4 ] =  -5.0625 ;  murs_xb[ 4 ] =  6 ; murs_yb[ 4 ] =  -3 ;
    murs_xa[ 5 ] =  6 ; murs_ya[ 5 ] =  -3 ;  murs_xb[ 5 ] =  6.5 ; murs_yb[ 5 ] =  0.5 ;
    murs_xa[ 6 ] =  6.5 ; murs_ya[ 6 ] =  0.5 ;  murs_xb[ 6 ] =  4.59375 ; murs_yb[ 6 ] =  3.4375 ;
    murs_xa[ 7 ] =  4.59375 ; murs_ya[ 7 ] =  3.4375 ;  murs_xb[ 7 ] =  0.5 ; murs_yb[ 7 ] =  6 ;
    murs_xa[ 8 ] =  0.5 ; murs_ya[ 8 ] =  6 ;  murs_xb[ 8 ] =  -1.5 ; murs_yb[ 8 ] =  6.5 ;
    murs_xa[ 9 ] =  -1.5 ; murs_ya[ 9 ] =  6.5 ;  murs_xb[ 9 ] =  -1.5 ; murs_yb[ 9 ] =  5 ;
    murs_xa[ 10 ] =  -1.5 ; murs_ya[ 10 ] =  5 ;  murs_xb[ 10 ] =  -4.5 ; murs_yb[ 10 ] =  5 ;
    murs_xa[ 11 ] =  -4.5 ; murs_ya[ 11 ] =  5 ;  murs_xb[ 11 ] =  -5.5 ; murs_yb[ 11 ] =  3.5 ;
    murs_xa[ 12 ] =  -5.5 ; murs_ya[ 12 ] =  3.5 ;  murs_xb[ 12 ] =  -5.5 ; murs_yb[ 12 ] =  -1.5 ;
//    ========              HARD CODED Map                              ===================================


    Rworld->drawMap(murs_xa,murs_ya, murs_xb, murs_yb);
    connect(ui->Rsivia,SIGNAL(drawArtRob(double,double)),this,SLOT(drawArtRobot(double,double)));
    connect(ui->Rsivia,SIGNAL(addPoint(double,double,bool)),this,SLOT(addPoint(double,double,bool)));

    // Verifie si une image a déja été calculée
    QFile img_filname("SIVIA.png");
    if (img_filname.exists()){
        qDebug("chargement de l'image");
        ui->Rsivia->Load(img_filname.fileName());
    }
}



void MainWindow::Init()
{


    epsilon=ui->EpsilonSpinBox->value();
    xv =0;   //pose vraie du robot
    yv =0;   // Elle est inconnue

    for(int i = 0; i < murs_xa.size(); i++){
        Rworld->DrawLine(murs_xa[i],murs_ya[i],murs_xb[i],murs_yb[i],QPen(Qt::red));
    }
    ui->Rsivia->update();

}

MainWindow::~MainWindow()  {delete ui;}




void MainWindow::drawRobotArticuler(double a1, double a2){
    double xv1 = xv+d1*cos(a1); double yv1 = yv+d1*sin(a1);
    double xv2 = xv1 + d2*cos(a1+a2); double yv2 = yv1 + d2*sin(a1+a2);
    double xv3 = xv2 + delta*cos(a1+a2); double yv3 = yv2 + delta*sin(a1+a2);

    QPen pen = QPen(Qt::blue);
    pen.setWidth(2);

    Rworld->DrawLine(xv,yv,xv1,yv1,pen);
    Rworld->DrawLine(xv1,yv1,xv2,yv2,pen);
    Rworld->DrawLine(xv2,yv2,xv3,yv3,QPen(Qt::green));
}

void MainWindow::drawArtRobot(double a1, double a2){
    //double d1 = 3; double d2 = 5;

    Rworld->Clean();
    for(uint i = 0; i < murs_xa.size(); i++){
        Rworld->DrawLine(murs_xa[i],murs_ya[i],murs_xb[i],murs_yb[i],QPen(Qt::red));
    }
    drawRobotArticuler(a1,a2);
    for(uint i = 0; i < L_a1.size(); i+=3){
        drawRobotArticuler(L_a1[i],L_a2[i]);
    }


    // On recharge le résulat du SIVIA et on trace le chemin
    ui->Rsivia->Clean();
    ui->Rsivia->Load("SIVIA.png");
    for(uint i = 1; i < L_a1.size(); i++){
        //r->DrawEllipse(L_a1[i],L_a2[i],0.2,QPen(Qt::black),QBrush(Qt::black));
        QPen pen = QPen(Qt::black);
        pen.setWidth(2);
        ui->Rsivia->DrawLine(L_a1[i-1],L_a2[i-1],L_a1[i],L_a2[i],pen);
    }
    ui->Rsivia->repaint();
}

void MainWindow::addPoint(double a1, double a2, bool add)
{
    //qDebug() << "add point "<<a1 << a2;
    if(add == true){
        L_a1.push_back(a1);
        L_a2.push_back(a2);
    } else {
        L_a1.pop_back();
        L_a2.pop_back();
    }

}

void MainWindow::on_ButtonStart_clicked()
{
    Init();
    ui->Rsivia->Clean();
    box X0(2);
    X0[1] = interval(sxmin, sxmax);
    X0[2] = interval(symin, symax);
    sivia->runSivia(&Sivia::separateur,X0, ui->EpsilonSpinBox->value());
    ui->Rsivia->Save("SIVIA");
}

void MainWindow::on_SaveImages_clicked()
{
    Rworld->Clean();
    for(uint i = 0; i < murs_xa.size(); i++){
        Rworld->DrawLine(murs_xa[i],murs_ya[i],murs_xb[i],murs_yb[i],QPen(Qt::red));
    }
    for(uint i = 0; i < L_a1.size(); i+=3){
        drawRobotArticuler(L_a1[i],L_a2[i]);
    }
    ui->Rsivia->Save("Resultats_Sivia");
    Rworld->Save("Resultats_Wolrd");
}
